/**
 * description
 *
 * @author eson
 * 2022年6月09日-16:29:17
 */
package com.yuandian.dataflow.grpc;

import java.time.Duration;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.google.protobuf.*;
import com.google.protobuf.util.JsonFormat;
import com.yuandian.common.Config;
import com.yuandian.dataflow.projo.Response;
import com.yuandian.dataflow.proto.CollectPacketsServerGrpc;


import com.yuandian.dataflow.proto.Base.Request;
import com.yuandian.dataflow.proto.CollectPacketsServerGrpc.CollectPacketsServerImplBase;

import com.yuandian.dataflow.proto.msgtype.*;

import io.grpc.ManagedChannelBuilder;
import lombok.var;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.*;
import org.springframework.web.client.RestTemplate;

/**
 * description
 *
 * @author eson
 * 2022年6月09日-16:29:17
 */
@Slf4j
public class CollectPackets extends CollectPacketsServerImplBase {


    public static void main(String[] args) throws Exception {

        Config.UseConfig((cnf) -> {
            var tid = cnf.get("test_id");
            log.info("config {}", tid);
            return null;
        });

        var managedChannelBuilder = ManagedChannelBuilder.forAddress("10.1.1.117", 60017);
        // var managedChannelBuilder = ManagedChannelBuilder.forAddress("127.0.0.1", 60017);
        var channel = managedChannelBuilder.maxInboundMessageSize(Integer.MAX_VALUE).usePlaintext().build();

        Map<String, Function<Any, Object>> domap = getHandleFuncMap();
        try {
            var stub = CollectPacketsServerGrpc.newBlockingStub(channel);
            var response = stub.getPackets(Request.newBuilder().setVersion("v1.0.0").build());

            // todo 优先处理 回溯分析的 其他的流先忽略
            String typeUrl = Any.pack(BacktrackingFlowOuterClass.BacktrackingFlow.getDefaultInstance()).getTypeUrl();

            int count = 10;
            while (count-- > 0) {
                var iter = response.next();
                Instant now = Instant.now();
                var i = 0;
                for (var p : iter.getPacketsList()) {
                    if (typeUrl.equals(p.getTypeUrl())) {
                        var func = domap.get(p.getTypeUrl());
                        if (func != null) {
                            func.apply(p);
                            i++;
                        }
                    } else {
                        log.info("不是回溯分析的流，暂时忽略: {}", p.getTypeUrl());
                    }
                }

                log.info("次序:{} 条数: {}, {}:ms", count, i, Duration.between(now, Instant.now()).toMillis());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            channel.shutdown().awaitTermination(1, TimeUnit.SECONDS);
        }

    }

    // todo 待优化
    private static HashMap<String, Function<Any, Object>> getHandleFuncMap() {
        return new HashMap<String, Function<Any, Object>>() {
            {
                /*put(Any.pack(ApmBaseDataFlowOuterClass.ApmBaseDataFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(ApmBaseDataFlowOuterClass.ApmBaseDataFlow.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));
                        
                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(AppFlowOuterClass.AppFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(AppFlowOuterClass.AppFlow.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });*/

                // todo 优先对接 回溯分析 的流，保存数据到 mongodb
                put(Any.pack(BacktrackingFlowOuterClass.BacktrackingFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(BacktrackingFlowOuterClass.BacktrackingFlow.class);
                        System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        RestTemplate client = new RestTemplate();
                        HttpHeaders headers = new HttpHeaders();
                        headers.setContentType(MediaType.APPLICATION_JSON);

                        HttpEntity<Object> requestEntity = new HttpEntity<>(JsonFormat.printer().print(result), headers);
                        String url = "http://localhost:3440/test";
                        ResponseEntity<Response> response = client.exchange(url, HttpMethod.POST, requestEntity, Response.class);

                        System.out.println("result:" + response.getBody());
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });

                /*put(Any.pack(BusinessBodyDataOuterClass.BusinessBodyData.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(BusinessBodyDataOuterClass.BusinessBodyData.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(BussFlowDbOuterClass.BussFlowDb.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(BussFlowDbOuterClass.BussFlowDb.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(BussFlowExternalOuterClass.BussFlowExternal.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(BussFlowExternalOuterClass.BussFlowExternal.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(BussFlowMiddOuterClass.BussFlowMidd.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(BussFlowMiddOuterClass.BussFlowMidd.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(BussFlowOrlOuterClass.BussFlowOrl.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(BussFlowOrlOuterClass.BussFlowOrl.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(DataFlowOuterClass.DataFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(DataFlowOuterClass.DataFlow.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(QoeFlowOuterClass.QoeFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(QoeFlowOuterClass.QoeFlow.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(SstFlowOuterClass.SstFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(SstFlowOuterClass.SstFlow.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });
                put(Any.pack(UsrFlowOuterClass.UsrFlow.getDefaultInstance()).getTypeUrl(), (p) -> {
                    try {
                        var result = p.unpack(UsrFlowOuterClass.UsrFlow.class);
                        // System.out.println(result.getClass().toString() + " ,val: " + JsonFormat.printer().print(result));

                        // MongodbTest.insertMsgToMongoDB(result);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    return null;
                });*/
            }
        };
    }

    private static String convert(com.google.protobuf.ByteString byteString) {
        ArrayList<String> res = new ArrayList<>();
        byteString.forEach(byteStr -> {
            res.add(String.valueOf(byteStr & 0xff));
        });
        return res.stream().collect(Collectors.joining("."));
    }
}
